//////////////////////////////////////////////////////////////////////////////////////////
// Project description
// 
// Name: Example Application
// Purpose: Example application of how to use touchlib in C++
//
// Author: Laurence Muller (aka Falcon4ever)
// Contact: info@falcon4ever.com
// Site: www.multigesture.net
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
// Include files
// 
#include "main.h"

// Used namespaces
using namespace std;

// Global variables
ITouchScreen *screen;	// TouchScreen device
ExampleApp myapp;		// Your application
bool running = true;	// Simple on/off switch for our main loop

int main(int argc, char **argv) 
{
	// Setup Touchlib
	screen = TouchScreenDevice::getTouchScreen();	// Assign the TouchScreen device
	//screen->setDebugMode(false);	// Enable/Disable touchlib debug mode, on default its set to true.

	if(!screen->loadConfig("config.xml"))	// Setup default values in no config.xml is available.
	{		
		string label;
		#ifdef WIN32
			label = screen->pushFilter("dsvlcapture");
		#else
			label = screen->pushFilter("cvcapture");		
			screen->setParameter(label, "source", "cam");
		#endif
		screen->setParameter(label, "source", "cam");
		screen->pushFilter("mono");
		screen->pushFilter("smooth");
		screen->pushFilter("backgroundremove");
		label = screen->pushFilter("brightnesscontrast");
			screen->setParameter(label, "brightness", "0.1");
			screen->setParameter(label, "contrast", "0.4");		
		label = screen->pushFilter("rectify");
			screen->setParameter(label, "level", "90");		
		screen->saveConfig("config.xml");
	}

	string bgLabel = screen->findFirstFilter("backgroundremove");
	string recLabel = screen->findFirstFilter("rectify");
	screen->registerListener((ITouchListener *)&myapp);	// Attach our application
	
	SLEEP(1000);
	screen->setParameter(bgLabel, "mask", (char*)screen->getCameraPoints());
	screen->setParameter(bgLabel, "capture", "");
	
	// Note: Begin processing should only be called after the screen is set up
	screen->beginProcessing();
	screen->beginTracking();

	// Main loop for your application
	do
	{
		int keypressed = cvWaitKey(32) & 255;

		if(keypressed != 255 && keypressed > 0)
		{
			printf("KP: %d\n", keypressed);
		}
        if( keypressed == 27) 
		{
			break;		// ESC = quit
		}
        if( keypressed == 98)				// b = recapture background
		{
			screen->setParameter(bgLabel, "capture", "");
			myapp.clearFingers();			
		}
        if( keypressed == 114)				// r = auto rectify..
		{
			screen->setParameter(recLabel, "level", "auto");
		}

		screen->getEvents();		// Update environment
		myapp.frame();

	} while( running );

	TouchScreenDevice::destroy();

	return 0;
}
